// 2013-01-28

// (C) John COLIBRI 2013
// documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_edit_stylehook_color;
  interface
    uses Vcl.StdCtrls, Vcl.Controls, Winapi.Messages;

    type TEditStyleHookColor=
             class(TEditStyleHook)
               strict private
                 procedure UpdateColors;
                 procedure do_display(p_text: String);
               protected
                 procedure WndProc(var Message: TMessage); override;
               public
                 constructor Create(AControl: TWinControl); override;
             end; // TEditStyleHookColor

  implementation
    Uses Winapi.Windows, Vcl.Graphics, Vcl.Themes, Vcl.Styles, SysUtils
        , Dialogs
        , u_display_simple
        , u_styles_helpers;

    type TWinControlClass= class(TWinControl);

    // -- TEditStyleHookColor

    constructor TEditStyleHookColor.Create(AControl: TWinControl);
      begin
        inherited;
        UpdateColors;
      end; // Create

    procedure TEditStyleHookColor.do_display(p_text: String);
      begin
        display(Format('%-6s %s', [Control.Name, p_text]));
      end; // do_display

    procedure TEditStyleHookColor.UpdateColors;
      var LStyle: TCustomStyleServices;
      begin
        if Control.Enabled
          then
            begin
              do_display('UpdateColors enabled');
              Brush.Color:= TWinControlClass(Control).Color;
              FontColor:= TWinControlClass(Control).Font.Color;
            end
          else
            begin
              do_display('UpdateColors disabled');
              LStyle:= StyleServices;

              Brush.Color:= LStyle.GetStyleColor(scEditDisabled);
              FontColor:= LStyle.GetStyleFontColor(sfEditBoxTextDisabled);
            end;
      end; // UpdateColors

    procedure TEditStyleHookColor.WndProc(var Message: TMessage);
      begin
        case Message.Msg of
          CN_CTLCOLORMSGBOX..CN_CTLCOLORSTATIC:
              begin
                do_display('cn '+ f_display_cn(Message.Msg));
                UpdateColors;

                SetTextColor(Message.WParam, ColorToRGB(FontColor));
                SetBkColor(Message.WParam, ColorToRGB(Brush.Color));
                Message.Result:= LRESULT(Brush.Handle);
                Handled:= True;
              end;
          CM_ENABLEDCHANGED:
              begin
                do_display('enabled_changed');
                UpdateColors;

                Handled:= False;
              end
          else inherited WndProc(Message);
        end;
      end; // WndProc

    end.// u_zzz
