// -- 2013-01-28

// -- (C) John COLIBRI 2007
// -- documentation: http://www.jcolibri.com

// -- some style utils

unit u_styles_helpers;
  interface
    uses Classes, Vcl.Controls, Vcl.ComCtrls
        , Generics.Collections
        ;

    const k_BDSUSERDIR= 'C:\Documents and Settings\All Users\Documents\RAD Studio\9.0\';

    procedure list_registered_styles(p_c_strings: tStrings);
    procedure list_exe_vsf_files(p_c_strings: tStrings);
    procedure list_BDSUSERDIR_vsf_files(p_c_strings: tStrings);
    procedure register_and_set_BDSUSERDIR_vsf_style(const pk_vsf_style_filename: string);
    procedure load_BDSUSERDIR_vsf_style(const pk_vsf_style_filename: string);

    procedure get_style_info(p_style_name: String;
        p_c_listview: tListView; p_c_imagelist: tImageList);

    function f_display_cn(p_cn: Word): String;

  implementation
    uses
        TypInfo, Graphics
        , System.SysUtils
        , System.IOUtils // tDirectory

        , Vcl.Styles, Vcl.Themes
        , u_display_simple
        ;

    procedure list_registered_styles(p_c_strings: tStrings);
        // -- fill ListBoxLoadedStyles with the names of styles
      Var l_style_name: string;
      begin
        p_c_strings.Clear;

        for l_style_name in TStyleManager.StyleNames do
          if p_c_strings.IndexOf(l_style_name)= - 1
            then p_c_strings.Add(l_style_name);
      end; // list_registered_styles

    procedure list_exe_vsf_files(p_c_strings: tStrings);
        // -- fill ListBoxDiskStyles with the names of styles from our style folder
      Var l_exe_dir, l_file_name: string;
      begin
        l_exe_dir:= IncludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0)));
        p_c_strings.Clear;

        for l_file_name in TDirectory.GetFiles(l_exe_dir+ 'Styles',
                '*'+ TStyleEngine.FileExtension) do
          p_c_strings.Add(ExtractFileName(l_file_name));
      end; // list_exe_vsf_files

    procedure list_BDSUSERDIR_vsf_files(p_c_strings: tStrings);
        // -- fill ListBoxDiskStyles with the names of styles from our style folder
      Var l_styles_dir, l_file_name: string;
      begin
        p_c_strings.Clear;
        l_styles_dir:= k_BDSUSERDIR+ 'Styles\';

        for l_file_name in TDirectory.GetFiles(l_styles_dir,
                '*'+ TStyleEngine.FileExtension) do
          p_c_strings.Add(ExtractFileName(l_file_name));
      end; // list_BDSUSERDIR_vsf_files

    procedure load_BDSUSERDIR_vsf_style(const pk_vsf_style_filename: string);
      begin
        if FileExists(pk_vsf_style_filename)
          then TStyleManager.LoadFromFile(pk_vsf_style_filename);
      end; // load_BDSUSERDIR_vsf_style

    procedure register_and_set_BDSUSERDIR_vsf_style(const pk_vsf_style_filename: string);
      begin
        if FileExists(pk_vsf_style_filename)
          then TStyleManager.SetStyle(TStyleManager.LoadFromFile(pk_vsf_style_filename));
      end; // register_and_set_BDSUSERDIR_vsf_style

    function f_display_cn(p_cn: Word): String;
      begin
        Result:= '';
        if p_cn= CN_CHARTOITEM
          then Result:= 'CharToItem' else
        if p_cn= CN_command
          then Result:= 'command' else
        if p_cn= CN_CompareItem
          then Result:= 'CompareItem' else
        if p_cn= CN_CtlColorBtn
          then Result:= 'CtlColorBtn' else
        if p_cn= CN_CtlColorDlg
          then Result:= 'CtlColorDlg' else
        if p_cn= CN_CtlColorEdit
          then Result:= 'CtlColorEdit' else
        if p_cn= CN_ctlColorListBox
          then Result:= 'ctlColorListBox' else

        if p_cn= CN_ctlColorMsgBox
          then Result:= 'ctlColorMsgBox' else
        if p_cn= CN_ctlColorScrollBar
          then Result:= 'ctlColorScrollBar' else
        if p_cn= CN_ctlColorStatic
          then Result:= 'ctlColorStatic' else
(*
        if p_cn= CN_xxx
          then Result:= 'xxx' else

  CN_DELETEITEM        = CN_BASE + WM_DELETEITEM;
  CN_DRAWITEM          = CN_BASE + WM_DRAWITEM;
  CN_HSCROLL           = CN_BASE + WM_HSCROLL;
  CN_MEASUREITEM       = CN_BASE + WM_MEASUREITEM;
  CN_PARENTNOTIFY      = CN_BASE + WM_PARENTNOTIFY;
  CN_VKEYTOITEM        = CN_BASE + WM_VKEYTOITEM;
  CN_VSCROLL           = CN_BASE + WM_VSCROLL;
  CN_KEYDOWN           = CN_BASE + WM_KEYDOWN;
  CN_KEYUP             = CN_BASE + WM_KEYUP;
  CN_CHAR              = CN_BASE + WM_CHAR;
  CN_SYSKEYDOWN        = CN_BASE + WM_SYSKEYDOWN;
  CN_SYSCHAR           = CN_BASE + WM_SYSCHAR;
  CN_NOTIFY            = CN_BASE + WM_NOTIFY;
*)
      end; // f_display_cn

    procedure get_style_info(p_style_name: String;
        p_c_listview: tListView; p_c_imagelist: tImageList);

      procedure create_array_bitmap(p_width, p_height: Word; p_color_array: Array of TColor;
          var pv_bitmap: TBitmap);
        Var l_width: integer;
            l_color_index: integer;
        begin
          with pv_bitmap do
          begin
            PixelFormat:= pf24bit;
            Width:= p_width;
            Height:= p_height;
            Canvas.Brush.Color:= clBlack;
            Canvas.FillRect(Rect(0, 0, p_width, p_height));
            l_width:= (p_width- 2) div (High(p_color_array)+ 1);
            for l_color_index:= 0 to High(p_color_array) do
            begin
              Canvas.Brush.Color:= p_color_array[l_color_index];
              Canvas.FillRect(Rect((l_width* l_color_index)+ 1, 1, l_width* (l_color_index+ 1)+ 1, p_height- 1))
            end;
          end; // with pv_bitmap
        end; // create_array_bitmap

      Var l_style_color: TStyleColor;
          l_color: TColor;
          l_c_bitmap: TBitmap;

      begin // get_style_info
        p_c_listview.Items.Clear;
        p_c_imagelist.Clear;

        for l_style_color:= scBorder to scWindow do
        begin
          l_color:= TStyleManager.Style[p_style_name].GetStyleColor(l_style_color);
          l_c_bitmap:= TBitmap.Create;
          try
            create_array_bitmap(16, 16, [l_color], l_c_bitmap);

            p_c_imagelist.Add(l_c_bitmap, nil);
            with p_c_listview.Items.Add do
            begin
              ImageIndex:= p_c_imagelist.Count- 1;
              Caption:= GetEnumName(TypeInfo(TStyleColor), Ord(l_style_color));
              SubItems.Add(ColorToString(l_color));
            end; // with p_c_listview.Items.Add
          finally
            l_c_bitmap.Free;
          end;
        end; // for l_style_color
      end; // get_style_info

  end.
